'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.release = exports.setAttribute = exports.isAttribute = exports.isElement = exports.makeElementNodeFromQtComponent = exports.makeElementNode = exports.makeAttributeNode = undefined;

var _get = require('lodash/get');

var _get2 = _interopRequireDefault(_get);

var _set = require('lodash/set');

var _set2 = _interopRequireDefault(_set);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var fakeRoot = void 0;
var realRoot = void 0;
var activeCounter = 0;

var getRoot = function getRoot(rootInstance) {
  if (fakeRoot) {
    return fakeRoot;
  }

  fakeRoot = Qt.createQmlObject('import QtQuick 2.7; Item { visible: false; }', rootInstance, 'FakeRoot');

  realRoot = rootInstance;

  return fakeRoot;
};

var ATTRIBUTE = 'ATTRIBUTE';
var ELEMENT = 'ELEMENT';

var createQmlObject = function createQmlObject(qmlContent, rootElem, name) {
  return Qt.createQmlObject(qmlContent, getRoot(rootElem), name);
};

var createQmlObjectFromQtComponent = function createQmlObjectFromQtComponent(component, rootElem, name) {
  return component.createObject(getRoot(rootElem));
};

var makeAttributeNode = exports.makeAttributeNode = function makeAttributeNode(name) {
  var value = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
  return void 0, {
    id: ++activeCounter,
    type: ATTRIBUTE,
    name: name,
    value: value,
    initialValue: null
  };
};

var makeElementNode = exports.makeElementNode = function makeElementNode(name, defaultProp, qmlContent, rootElem) {
  return void 0, {
    id: ++activeCounter,
    type: ELEMENT,
    name: name,
    defaultProp: defaultProp,
    children: [],
    value: createQmlObject(qmlContent, rootElem, name)
  };
};

var makeElementNodeFromQtComponent = exports.makeElementNodeFromQtComponent = function makeElementNodeFromQtComponent(name, defaultProp, component, rootElem) {
  return void 0, {
    id: ++activeCounter,
    type: ELEMENT,
    name: name,
    defaultProp: defaultProp,
    children: [],
    value: createQmlObjectFromQtComponent(component, rootElem, name)
  };
};

var isElement = exports.isElement = function isElement(obj) {
  return obj.type && obj.type === ELEMENT;
};
var isAttribute = exports.isAttribute = function isAttribute(obj) {
  return obj.type && obj.type === ATTRIBUTE;
};

var arrify = function arrify(v) {
  return Array.isArray(v) ? v : [v];
};

var setAttribute = exports.setAttribute = function setAttribute(parent, key, children) {
  var qmlObject = parent.value;
  var prev = (0, _get2.default)(qmlObject, key);

  if (prev && typeof prev.push === 'function') {
    if (typeof prev.clear === 'function') {
      prev.clear();
    }
    arrify(children).forEach(function (child) {
      prev.push(child.value);
    });
  } else {

    (0, _set2.default)(qmlObject, key, children.value.value);
  }
};

var release = exports.release = function release(elem) {
  elem.destroy();
};